/*
 * Caudium - An extensible World Wide Web server
 * Copyright  2000-2002 The Caudium Group
 * Based on Pike Protocols.SMTP source code.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
/*
 * $Id: ESMTP.pmod,v 1.4.6.3 2002/04/25 14:35:31 kiwi Exp $
 */

//! file: etc/modules/CAMAS.pmod/ESMTP.pmod
//!  This Camas Pike module handle SMTP protocol with some extensions
//!  of the SMTP protocol like DSN
//! cvs_version: $Id: ESMTP.pmod,v 1.4.6.3 2002/04/25 14:35:31 kiwi Exp $

//! class: ESMTP.protocol
//!  This is basic function used to connect to server.
//! scope: private

class protocol
{
  // Maybe this should be the other way around?
  inherit Protocols.NNTP.protocol;
}

//! class: ESMTP.client
//!  This class is used to send mail using SMTP protocol.
//!  The calls are more or less compatible with Pike's Protocols.SMTP classes.

class client
{
  inherit protocol;

  int is_esmtp;

  constant DSN_SUCCESS = 1;
  constant DSN_DELAY   = 2;
  constant DSN_FAILURE = 4;

  constant reply_codes =
  ([ 211:"System status, or system help reply",
     214:"Help message",
     220:"<host> Service ready",
     221:"<host> Service closing transmission channel",
     250:"Requested mail action okay, completed",
     251:"User not local; will forward to <forward-path>",
     354:"Start mail input; end with <CRLF>.<CRLF>",
     421:"<host> Service not available, closing transmission channel "
         "[This may be a reply to any command if the service knows it "
         "must shut down]",
     450:"Requested mail action not taken: mailbox unavailable "
         "[E.g., mailbox busy]",
     451:"Requested action aborted: local error in processing",
     452:"Requested action not taken: insufficient system storage",
     500:"Syntax error, command unrecognized "
         "[This may include errors such as command line too long]",
     501:"Syntax error in parameters or arguments",
     502:"Command not implemented",
     503:"Bad sequence of commands",
     504:"Command parameter not implemented",
     550:"Requested action not taken: mailbox unavailable "
         "[E.g., mailbox not found, no access]",
     551:"User not local; please try <forward-path>",
     552:"Requested mail action aborted: exceeded storage allocation",
     553:"Requested action not taken: mailbox name not allowed "
         "[E.g., mailbox syntax incorrect]",
     554:"Transaction failed" ]);

  static private int cmd(string c, string|void comment)
  {
    int r = command(c);
    switch(r) {
    case 200..399:
      break;
    default:
      throw(({"SMTP: "+c+"\n"+(comment?"SMTP: "+comment+"\n":"")+
	      "SMTP: "+reply_codes[r]+"\n", backtrace()}));
    }
    return r;
  }

  void create(void|string server, int|void port, void|string maildomain)
  {
    if(!server)
    {
      // Lookup MX record here (Using DNS.pmod)
      object dns=master()->resolv("Protocols")["DNS"]->client();
      server=dns->get_primary_mx(gethostname());
    }

    if(!port)
      port = 25;

    if(!server || !connect(server, port))
      {
	throw(({"Failed to connect to mail server.\n",backtrace()}));
      }

    if(readreturncode()/100 != 2)
      throw(({"Connection refused by SMTP server.\n",backtrace()}));

    string fqdn = gethostname ();
    fqdn += (maildomain) ? ("." + maildomain) : "";
    if(catch(cmd("EHLO "+fqdn))) {
      is_esmtp = 0;
      cmd("HELO "+fqdn, "greeting failed.");
    }
    else {
      is_esmtp = 1;
    }
  }

  void send_message(string from, array(string) to, string body, void|int dsn_options)
  {
    string dsn_comm = "";
    if (is_esmtp && dsn_options) {
      if (dsn_options & DSN_DELAY) {
	dsn_comm = "DELAY";
      }
      if (dsn_options & DSN_SUCCESS) {
	dsn_comm += (sizeof(dsn_comm)>0?",":"")+"SUCCESS";
      }
      if (dsn_options & DSN_FAILURE) {
	dsn_comm += (sizeof(dsn_comm)>0?",":"")+"FAILURE";
      }
    }
    if (is_esmtp && dsn_options) {
      cmd("MAIL FROM: <" + from + "> RET=HDRS ENVID=IMHOMSG");
    }
    else {
      cmd("MAIL FROM: <" + from + ">");
    }
    if (!is_esmtp || !sizeof (dsn_comm)) {
      foreach(to, string t) {
	cmd("RCPT TO: <" + t + ">");
      }
    }
    else {
      foreach(to, string t) {
	cmd("RCPT TO: <" + t + "> NOTIFY=" + dsn_comm + " ORCPT=rfc822;" + t);
      }
    }
    cmd("DATA");
    cmd(body+"\r\n.");
    cmd("QUIT");
  }
}
